﻿using System;
using System.Diagnostics;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    /// <returns>태스크</returns>
    static async Task Main()
    {
        string installerURL      = "https://www.python.org/ftp/python/3.12.4/python-3.12.4-amd64.exe";
        string installerFilePath = @"C:\temp\python_installer.exe";

        if(!Directory.Exists(@"C:\temp"))
        {
            Directory.CreateDirectory(@"C:\temp");
        }

        Console.WriteLine($"[{DateTime.Now.ToString("HH:mm:ss")}] 파이썬 설치 프로그램 다운로드를 시작합니다.");

        await DownloadFileAsync(installerURL, installerFilePath);

        Console.WriteLine($"[{DateTime.Now.ToString("HH:mm:ss")}] 파이썬 설치 프로그램 다운로드를 종료합니다.");

        Console.WriteLine($"[{DateTime.Now.ToString("HH:mm:ss")}] 파이썬 프로그램 설치를 시작합니다.");

        ProcessStartInfo processStartInfo = new()
        {
            FileName        = installerFilePath,
            Arguments       = "/quiet InstallAllUsers=1 PrependPath=1", // 자동 설치를 위한 인자
            UseShellExecute = false
        };

        Process process = new()
        {
            StartInfo = processStartInfo
        };

        process.Start();

        process.WaitForExit();

        Console.WriteLine($"[{DateTime.Now.ToString("HH:mm:ss")}] 파이썬 프로그램 설치를 종료합니다.");

        string pythonFilePath = @"C:\Program Files\Python312\python.exe";

        if(File.Exists(pythonFilePath))
        {
            Console.WriteLine("Python이 성공적으로 설치되었습니다.");
        }
        else
        {
            Console.WriteLine("Python 설치에 실패했습니다.");
        }

        Console.WriteLine("프로그램을 종료하기 위해 아무 키나 눌러 주세요.");

        Console.ReadKey(false);
    }

    #endregion

    #region 파일 다운로드하기 (비동기) - DownloadFileAsync(url, filePath)

    /// <summary>
    /// 파일 다운로드하기 (비동기)
    /// </summary>
    /// <param name="url">URL</param>
    /// <param name="filePath">파일 경로</param>
    /// <returns>처리 결과</returns>
    private static async Task<bool> DownloadFileAsync(string url, string filePath)
    {
        using HttpClient client = new HttpClient();

        try
        {
            using HttpResponseMessage response = await client.GetAsync(url, HttpCompletionOption.ResponseHeadersRead);

            response.EnsureSuccessStatusCode();

            using Stream stream = await response.Content.ReadAsStreamAsync();

            using FileStream fileStream = new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.None);

            await stream.CopyToAsync(fileStream);

            return true;
        }
        catch(HttpRequestException)
        {
            return false;
        }
    }

    #endregion
}